package geniusweb.bidspace;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertTrue;

import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.Map;

import org.junit.Test;

import geniusweb.bidspace.AllBidsList;
import geniusweb.issuevalue.Bid;
import geniusweb.issuevalue.DiscreteValue;
import geniusweb.issuevalue.DiscreteValueSet;
import geniusweb.issuevalue.Domain;
import geniusweb.issuevalue.NumberValue;
import geniusweb.issuevalue.NumberValueSet;
import geniusweb.issuevalue.ValueSet;

public class AllBidsListTest {
	private static final DiscreteValue I1V2 = new DiscreteValue("i1v2");
	private static final DiscreteValue I1V1 = new DiscreteValue("i1v1");
	private static final NumberValue I2V1 = new NumberValue("2.00");
	private static final NumberValue I2V2 = new NumberValue("2.45");
	private static final NumberValue I2V3 = new NumberValue("2.90");
	private static final String DOMAINNAME = "testdomain";
	private static final String ISSUE1 = "issue1";
	private static final String ISSUE2 = "issue2";
	private static final Map<String, ValueSet> issues = new HashMap<>();

	private static final ValueSet values1;
	private static final ValueSet values2;
	private static final BigDecimal TWO = new BigDecimal("2");
	private static final BigDecimal THREE = new BigDecimal("3");

	private static final Domain domain;

	private static final AllBidsList allBidsList;

	static {
		Collection<DiscreteValue> discretevalues1 = new LinkedList<>();
		discretevalues1.add(I1V1);
		discretevalues1.add(I1V2);
		values1 = new DiscreteValueSet(discretevalues1);
		issues.put(ISSUE1, values1);

		values2 = new NumberValueSet(TWO, THREE, new BigDecimal("0.45"));
		issues.put(ISSUE2, values2);

		domain = new Domain(DOMAINNAME, issues);
		allBidsList = new AllBidsList(domain);

	}

	@Test
	public void testListElements() {
		assertEquals(new BigInteger("6"), allBidsList.size());
		checkBid(allBidsList.get(0), I1V1, I2V1);
		checkBid(allBidsList.get(1), I1V1, I2V2);
		checkBid(allBidsList.get(2), I1V1, I2V3);
		checkBid(allBidsList.get(3), I1V2, I2V1);
		checkBid(allBidsList.get(4), I1V2, I2V2);
		checkBid(allBidsList.get(5), I1V2, I2V3);

	}

	@Test
	public void testiterator() {
		Iterator<Bid> iterator = allBidsList.iterator();
		checkNext(iterator, I1V1, I2V1);
		checkNext(iterator, I1V1, I2V2);
		checkNext(iterator, I1V1, I2V3);
		checkNext(iterator, I1V2, I2V1);
		checkNext(iterator, I1V2, I2V2);
		checkNext(iterator, I1V2, I2V3);
		assertFalse(iterator.hasNext());
	}

	private void checkNext(Iterator<Bid> iterator, DiscreteValue value1, NumberValue value2) {
		assertTrue(iterator.hasNext());
		checkBid(iterator.next(), value1, value2);
	}

	private void checkBid(Bid bid, DiscreteValue value1, NumberValue value2) {
		assertEquals(value1, bid.getValue(ISSUE1));
		assertEquals(value2, bid.getValue(ISSUE2));
	}
}
